/*****
*
* This file is part of the OVPlayer program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <gtk/gtk.h>
#include "SDL.h"
#include <vorbis/vorbisfile.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

GtkWidget *file_selector;
void store_filename(GtkFileSelection *selector, gpointer user_data);
OggVorbis_File ogg_file;
int of_valid = 0;
SDL_AudioSpec *sdlau;
int current_section, ptimer, stopped;

gint UpdateProgressTimer(gpointer gdata);
int mdown = 0;

gboolean sb_mdown(GtkWidget *wid, GdkEventButton *ev, gpointer d) {
  mdown = 1;
  gtk_timeout_remove(ptimer);
  return TRUE;
}

gboolean sb_mup(GtkWidget *wid, GdkEventButton *ev, gpointer d) {
  if (mdown) {
    mdown = 0;
    ptimer = gtk_timeout_add(500, UpdateProgressTimer, NULL);
  }
  return TRUE;
}

void hscrollbar1_value_changed(GtkAdjustment *adjustment, gpointer user_data) {
  if (of_valid) {
    SDL_PauseAudio(1);
    ov_time_seek(&ogg_file,ov_adj->value);
    SDL_PauseAudio(0);
  }
}

void oggsdl_audio_cb(void *userdata, Uint8 *stream, int len) {
  int ret, rem, eos;
  Sint16 *sample;
  Sint16 maxL, maxR;
  rem = len;
  eos = 0;
  sample = (Sint16*)stream;
  while (rem) {
    ret = ov_read((OggVorbis_File *)userdata, stream, rem, 0, 2, 1, &current_section);
    rem = rem - ret;
    if (ret == 0) {
      rem = 0;
      eos = 1;
    } else if (ret < 0) {
      fprintf(stderr, "\nError: ");
      switch (ret) {
	case OV_HOLE :
	  fprintf(stderr, "OV_HOLE : Interruption of Data\n");
	  break;
	case OV_EBADLINK :
	  fprintf(stderr, "OV_EBADLINK : Invalid Stream Section, or Link is Corrupt\n");
	  break;
      }
    }
    stream = stream + ret;
  }
  maxL = maxR = 0;
  /* at this point we search through the buffer for the highest amplitude, Left
     and Right.  This is somewhat a problem as ov_read may return multiple
     channels where SDL expects only two, but I don't care right now. */
  /* at the very least, I will check to see if this is a mono audio source,
     and adjust as needed. */
  for (rem = 0; rem < len; rem += 4) {
    if (maxL < *sample) maxL = *sample;
    sample++;
    if (ov_info((OggVorbis_File *)userdata, -1)->channels == 1) {
      maxR = maxL;
      rem -= 2;
    } else {
      if (maxR < *sample) maxR = *sample;
      sample++;
    }
  }

  SetLEDs(maxL / 2730,maxR / 2730);
  if (eos) {
    stopped = 1;
    SDL_PauseAudio(1);
    ov_raw_seek(&ogg_file,0);
    gtk_timeout_remove(ptimer);
    gtk_signal_handler_block_by_func(GTK_OBJECT(ov_adj),
                                     GTK_SIGNAL_FUNC(hscrollbar1_value_changed),
                                     NULL);
    gtk_adjustment_set_value(ov_adj,0.0);
    gtk_signal_handler_unblock_by_func(GTK_OBJECT(ov_adj),
                                       GTK_SIGNAL_FUNC(hscrollbar1_value_changed),
                                       NULL);
  }
}

void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  /* display an open file dialog filtering for .ogg files. */
  /* if no file is chosen simply exit this callback. */
  file_selector = gtk_file_selection_new("Select Ogg Vorbis file");
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
                      "clicked", GTK_SIGNAL_FUNC (store_filename), NULL);
                        
/* Ensure that the dialog box is destroyed when the user clicks a button. */

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
                                         "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
                                         (gpointer) file_selector);

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
                                         "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
                                         (gpointer) file_selector);

/* Display that dialog */

  gtk_widget_show (file_selector);
}

int stop_timer;

void StopComments();

gint stop_check (gpointer data) {
  if (stopped) {
    PlayPause(1);
    StopComments();
    SDL_PauseAudio(1);
    stopped = 0;
  }
  return 1;
}

gint UpdateProgressTimer(gpointer gdata) {
  gtk_signal_handler_block_by_func(GTK_OBJECT(ov_adj),
                                   GTK_SIGNAL_FUNC(hscrollbar1_value_changed),
                                   NULL);
  ov_adj->value = ov_time_tell(&ogg_file);
  gtk_adjustment_value_changed(ov_adj);
  gtk_signal_handler_unblock_by_func(GTK_OBJECT(ov_adj),
                                     GTK_SIGNAL_FUNC(hscrollbar1_value_changed),
                                     NULL);
  return 1;
}

guint SB_Context;
guint commentnumber;
int ctimer;

gint do_comment (gpointer data) {
  if (commentnumber) {
    gtk_statusbar_pop(GTK_STATUSBAR(statusbar1),SB_Context);
  }
  commentnumber++;
  if (ov_comment(&ogg_file,-1) == NULL)
    return 0;  /* we no longer want this timer to be called if the file is gone */
  if (stopped) return 0;
  if (ov_comment(&ogg_file,-1)->comments == 0) return 0;
  if (commentnumber > ov_comment(&ogg_file,-1)->comments)
    commentnumber = 1;
  gtk_statusbar_push(GTK_STATUSBAR(statusbar1),SB_Context,
                     ov_comment(&ogg_file,-1)->user_comments[commentnumber-1]);
  return 1;
}

void StartComments() {
  ctimer = gtk_timeout_add(2500,do_comment,NULL);
  commentnumber = 0;
  do_comment(NULL);
}

void StopComments() {
  gtk_timeout_remove(ctimer);
  gtk_statusbar_pop(GTK_STATUSBAR(statusbar1),SB_Context);
}

void store_filename(GtkFileSelection *selector, gpointer user_data) {
  FILE *f;

  if (of_valid) {
    SDL_PauseAudio(1);
    ov_clear(&ogg_file);
    StopComments();
    gtk_timeout_remove(stop_timer);
    SDL_CloseAudio();
  }

  f = fopen(gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_selector)),"rb");
  if (f == NULL) return;
  if (ov_open(f, &ogg_file,NULL,0) < 0) return;
  sdlau = (SDL_AudioSpec *)malloc(sizeof(SDL_AudioSpec));
  sdlau->format = AUDIO_S16;
  sdlau->samples = 2048;
  sdlau->callback = oggsdl_audio_cb;
  sdlau->userdata = (void *)&ogg_file;
  {
    vorbis_info *vi = ov_info(&ogg_file, -1);
    sdlau->freq = vi->rate;
    sdlau->channels = vi->channels;
  }
  if (SDL_OpenAudio(sdlau,NULL) < 0) return;
  SDL_PauseAudio(0);
  ov_adj->upper = ov_time_total(&ogg_file,-1);
  gtk_adjustment_changed(ov_adj);
  ptimer = gtk_timeout_add(500, UpdateProgressTimer, NULL);
  of_valid = 1;
  stopped = 0;
  stop_timer = gtk_timeout_add(50,stop_check,NULL);
  PlayPause(0);

  StartComments();
}


void autostart(char * fn) {
  FILE *f;

  f = fopen(fn,"rb");
  if (f == NULL) return;
  if (ov_open(f, &ogg_file,NULL,0) < 0) return;
  sdlau = (SDL_AudioSpec *)malloc(sizeof(SDL_AudioSpec));
  sdlau->format = AUDIO_S16;
  sdlau->samples = 2048;
  sdlau->callback = oggsdl_audio_cb;
  sdlau->userdata = (void *)&ogg_file;
  {
    vorbis_info *vi = ov_info(&ogg_file, -1);
    sdlau->freq = vi->rate;
    sdlau->channels = vi->channels;
  }
  if (SDL_OpenAudio(sdlau,NULL) < 0) return;
  SDL_PauseAudio(0);
  ov_adj->upper = ov_time_total(&ogg_file,-1);
  gtk_adjustment_changed(ov_adj);
  ptimer = gtk_timeout_add(500, UpdateProgressTimer, NULL);
  of_valid = 1;
  stopped = 0;
  stop_timer = gtk_timeout_add(50,stop_check,NULL);
  PlayPause(0);

  StartComments();
}

void
on_close1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  SDL_PauseAudio(1);
  ov_clear(&ogg_file);
  gtk_main_quit();

}


void
on_prevtrack_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  /* currently unimplemented, will be implemented after playlists */

}


int rtimer, ftimer;
double f_time;


gint rewind_timer (gpointer data) {
  if (!of_valid) return 0;

  SDL_PauseAudio(1);
  ov_time_seek(&ogg_file,f_time - 1.0);
  SDL_PauseAudio(0);

  f_time = f_time - 1.0;
  return 1;
}

gint fforward_timer (gpointer data) {
  if (!of_valid) return 0;

  SDL_PauseAudio(1);
  ov_time_seek(&ogg_file,f_time + 1.0);
  SDL_PauseAudio(0);

  f_time = f_time + 1.0;
  return 1;
}


void
on_rewind_pressed                      (GtkButton       *button,
                                        gpointer         user_data)
{
  f_time = ov_time_tell(&ogg_file);
  rtimer = gtk_timeout_add(100, rewind_timer, NULL);
}


void
on_rewind_released                     (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_timeout_remove(rtimer);
}


void
on_playpause_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  if (!of_valid) return;
  if (SDL_GetAudioStatus() == SDL_AUDIO_PLAYING) {
    PlayPause(1);
    SDL_PauseAudio(1);
    gtk_timeout_remove(ptimer);
    StopComments();
  } else {
    PlayPause(0);
    SDL_PauseAudio(0);
    ptimer = gtk_timeout_add(100, UpdateProgressTimer, NULL);
    stopped = 0;
    StartComments();
  }
}


void
on_stop_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
  stopped = 1;
  SDL_PauseAudio(1);
  ov_raw_seek(&ogg_file,0);
  gtk_timeout_remove(ptimer);
  gtk_adjustment_set_value(ov_adj,0.0);
  StopComments();
}


void
on_fforward_pressed                    (GtkButton       *button,
                                        gpointer         user_data)
{
  f_time = ov_time_tell(&ogg_file);
  ftimer = gtk_timeout_add(100, fforward_timer, NULL);
}


void
on_fforward_released                   (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_timeout_remove(ftimer);
}


void
on_nexttrack_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{

}

