/*****
*
* This file is part of the OVPlayer program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "SDL.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "xpms.h"

GtkAdjustment *ov_adj;
GtkWidget *hscrollbar1;

GtkWidget*
xpm2widget (GtkWidget *win, gchar ** xpm)
{
  GdkBitmap *mask;
  GdkPixmap *pixmap_data;
  GtkWidget *pixmap_widget;

  pixmap_data = gdk_pixmap_create_from_xpm_d(win->window,&mask,NULL,xpm);

  pixmap_widget = gtk_pixmap_new(pixmap_data, mask);

  gtk_widget_show(pixmap_widget);

  return pixmap_widget;
}

GtkWidget *PlayWidget, *PauseWidget;
GtkWidget *button3;
GtkWidget *ovplayer;

void
PlayPause (int n)
{
  static int first = 1;
  if (first) {
    gtk_container_add(GTK_CONTAINER(button3),PlayWidget);
    first = 0;
  } else if (n) { /* Display the Pause Pixmap */
    gtk_container_remove(GTK_CONTAINER(button3),PauseWidget);
    gtk_container_add(GTK_CONTAINER(button3),PlayWidget);
  } else { /* Display the Play Pixmap */
    gtk_container_remove(GTK_CONTAINER(button3),PlayWidget);
    gtk_container_add(GTK_CONTAINER(button3),PauseWidget);
  }
}

GtkWidget *leds;
GdkPixmap *gd, *gl, *rd, *rl;
Sint16 curL, curR;

#define SETLEDS_V1
void SetupLEDs(void) {
  gl = gdk_pixmap_create_from_xpm_d(ovplayer->window,NULL,NULL,LEDGreenLight_xpm);
  gd = gdk_pixmap_create_from_xpm_d(ovplayer->window,NULL,NULL,LEDGreenDark_xpm);
  rl = gdk_pixmap_create_from_xpm_d(ovplayer->window,NULL,NULL,LEDRedLight_xpm);
  rd = gdk_pixmap_create_from_xpm_d(ovplayer->window,NULL,NULL,LEDRedDark_xpm);
#ifdef SETLEDS_V1
  fprintf(stderr, "\n#define SETLEDS_V1\n");
#else
  fprintf(stderr, "\n#undef SETLEDS_V1\n");
#endif
}

#ifdef SETLEDS_V1
  GdkEvent ev;
#else
  GdkRectangle area;
#endif

void SetLEDs(Sint16 L, Sint16 R) {
  curL = L;
  curR = R;
#ifdef SETLEDS_V1
  ev.expose.type = GDK_EXPOSE;
  ev.expose.window = leds->window;
  ev.expose.send_event = FALSE;
  ev.expose.area.x = 0;
  ev.expose.area.y = 0;
  ev.expose.area.width = 32;
  ev.expose.area.height = 96;
  ev.expose.count = 0;

  /* I know, I know... I'm not SUPPOSED to call this function... but it works.
     AND I have not yet found an equally responsive alternative, nor does this
     crash.  So unless someone can produce equal results using "sanctioned"
     methods, this is the way the leds are updated.  -jjs */
  gtk_main_do_event(&ev);
#else
  area.x = 0;
  area.y = 0;
  area.width = 32;
  area.height = 96;

  gtk_widget_draw(leds,&area);
#endif
}

gboolean leds_exposed (GtkWidget *widget, GdkEventExpose *event, gpointer data) {
  int j;
  GdkGC *gc;
  gc = gdk_gc_new(widget->window);

  gdk_gc_set_clip_rectangle (gc, &event->area);

  for (j = 0; j < 8; j++) {
    if (j < curL)
      gdk_draw_pixmap(widget->window,gc,gl,
                      0,0,0,88 - j * 8,-1,-1);
    else
      gdk_draw_pixmap(widget->window,gc,gd,
                      0,0,0,88 - j * 8,-1,-1);
    if (j < curR)
      gdk_draw_pixmap(widget->window,gc,gl,
                      0,0,16,88 - j * 8,-1,-1);
    else
      gdk_draw_pixmap(widget->window,gc,gd,
                      0,0,16,88 - j * 8,-1,-1);
  }
  for (j = 8; j < 12; j++) {
    if (j < curL)
      gdk_draw_pixmap(widget->window,gc,rl,
                      0,0,0,88 - j * 8,-1,-1);
    else
      gdk_draw_pixmap(widget->window,gc,rd,
                      0,0,0,88 - j * 8,-1,-1);
    if (j < curR)
      gdk_draw_pixmap(widget->window,gc,rl,
                      0,0,16,88 - j * 8,-1,-1);
    else
      gdk_draw_pixmap(widget->window,gc,rd,
                      0,0,16,88 - j * 8,-1,-1);
  }

  gdk_gc_destroy(gc);
  return TRUE;
}

GtkWidget *statusbar1;

GtkWidget*
create_ovplayer (void)
{
  GtkWidget *vbox1;
  GtkWidget *menubar1;
  GtkWidget *file1;
  GtkWidget *file1_menu;
  GtkAccelGroup *file1_menu_accels;
  GtkWidget *open1;
  GtkWidget *close1;
  GtkWidget *hbox1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button4;
  GtkWidget *button5;
  GtkWidget *button6;
  GtkWidget *hbox2;

  ovplayer = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (ovplayer, "ovplayer");
  gtk_object_set_data (GTK_OBJECT (ovplayer), "ovplayer", ovplayer);
  gtk_window_set_title (GTK_WINDOW (ovplayer), "Ogg Vorbis Player");

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);
  gtk_widget_set_name (hbox2, "hbox2");
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "hbox2", hbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (ovplayer), hbox2);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox1, "vbox1");
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (hbox2), vbox1, FALSE, FALSE, 0);

  leds = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(leds),32,96);
  SetupLEDs();
  gtk_widget_show(leds);
  gtk_box_pack_start (GTK_BOX (hbox2), leds, FALSE, FALSE, 0);

  menubar1 = gtk_menu_bar_new ();
  gtk_widget_set_name (menubar1, "menubar1");
  gtk_widget_ref (menubar1);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "menubar1", menubar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menubar1);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar1, FALSE, FALSE, 0);

  file1 = gtk_menu_item_new_with_label ("File");
  gtk_widget_set_name (file1, "file1");
  gtk_widget_ref (file1);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "file1", file1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (file1);
  gtk_container_add (GTK_CONTAINER (menubar1), file1);

  file1_menu = gtk_menu_new ();
  gtk_widget_set_name (file1_menu, "file1_menu");
  gtk_widget_ref (file1_menu);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "file1_menu", file1_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (file1), file1_menu);
  file1_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (file1_menu));

  open1 = gtk_menu_item_new_with_label ("Open");
  gtk_widget_set_name (open1, "open1");
  gtk_widget_ref (open1);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "open1", open1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (open1);
  gtk_container_add (GTK_CONTAINER (file1_menu), open1);

  close1 = gtk_menu_item_new_with_label ("Close");
  gtk_widget_set_name (close1, "close1");
  gtk_widget_ref (close1);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "close1", close1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (close1);
  gtk_container_add (GTK_CONTAINER (file1_menu), close1);

  ov_adj = GTK_ADJUSTMENT(gtk_adjustment_new (0, 0, 1000000, 10, 60, 1));
  hscrollbar1 = gtk_hscrollbar_new (ov_adj);
  gtk_widget_set_name (hscrollbar1, "hscrollbar1");
  gtk_widget_ref (hscrollbar1);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "hscrollbar1", hscrollbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_range_set_update_policy(GTK_RANGE(hscrollbar1),
                             (GtkUpdateType)(GTK_UPDATE_DELAYED));
  gtk_widget_show (hscrollbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), hscrollbar1, TRUE, TRUE, 0);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox1, "hbox1");
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

  PlayWidget = xpm2widget(ovplayer,Play);
  gtk_widget_ref(PlayWidget);
  PauseWidget = xpm2widget(ovplayer,Pause);
  gtk_widget_ref(PauseWidget);

  button1 = gtk_button_new ();
  gtk_container_add(GTK_CONTAINER(button1),xpm2widget(ovplayer,PrevTrack));
  gtk_widget_set_name (button1, "button1");
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (hbox1), button1, FALSE, FALSE, 0);
  gtk_widget_set_usize (button1, 35, -2);

  button2 = gtk_button_new ();
  gtk_container_add(GTK_CONTAINER(button2),xpm2widget(ovplayer,Rewind));
  gtk_widget_set_name (button2, "button2");
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button2);
  gtk_box_pack_start (GTK_BOX (hbox1), button2, FALSE, FALSE, 0);
  gtk_widget_set_usize (button2, 35, -2);

  button3 = gtk_button_new ();
  PlayPause(1);
  gtk_widget_set_name (button3, "button3");
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button3);
  gtk_box_pack_start (GTK_BOX (hbox1), button3, FALSE, FALSE, 0);
  gtk_widget_set_usize (button3, 35, -2);

  button4 = gtk_button_new ();
  gtk_container_add(GTK_CONTAINER(button4),xpm2widget(ovplayer,Stop));
  gtk_widget_set_name (button4, "button4");
  gtk_widget_ref (button4);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "button4", button4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button4);
  gtk_box_pack_start (GTK_BOX (hbox1), button4, FALSE, FALSE, 0);
  gtk_widget_set_usize (button4, 35, -2);

  button5 = gtk_button_new ();
  gtk_container_add(GTK_CONTAINER(button5),xpm2widget(ovplayer,FastForward));
  gtk_widget_set_name (button5, "button5");
  gtk_widget_ref (button5);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "button5", button5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox1), button5, FALSE, FALSE, 0);
  gtk_widget_set_usize (button5, 35, -2);

  button6 = gtk_button_new ();
  gtk_container_add(GTK_CONTAINER(button6),xpm2widget(ovplayer,NextTrack));
  gtk_widget_set_name (button6, "button6");
  gtk_widget_ref (button6);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "button6", button6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button6);
  gtk_box_pack_start (GTK_BOX (hbox1), button6, FALSE, FALSE, 0);
  gtk_widget_set_usize (button6, 35, -2);

  statusbar1 = gtk_statusbar_new ();
  gtk_widget_set_name (statusbar1, "statusbar1");
  gtk_widget_ref (statusbar1);
  gtk_object_set_data_full (GTK_OBJECT (ovplayer), "statusbar1", statusbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (statusbar1);
  SB_Context = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar1),"Comments");
  gtk_box_pack_start (GTK_BOX (vbox1), statusbar1, FALSE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (open1), "activate",
                      GTK_SIGNAL_FUNC (on_open1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (close1), "activate",
                      GTK_SIGNAL_FUNC (on_close1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (on_prevtrack_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button2), "pressed",
                      GTK_SIGNAL_FUNC (on_rewind_pressed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button2), "released",
                      GTK_SIGNAL_FUNC (on_rewind_released),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button3), "clicked",
                      GTK_SIGNAL_FUNC (on_playpause_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button4), "clicked",
                      GTK_SIGNAL_FUNC (on_stop_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "pressed",
                      GTK_SIGNAL_FUNC (on_fforward_pressed),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "released",
                      GTK_SIGNAL_FUNC (on_fforward_released),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button6), "clicked",
                      GTK_SIGNAL_FUNC (on_nexttrack_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ov_adj), "value-changed",
		      GTK_SIGNAL_FUNC (hscrollbar1_value_changed),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (hscrollbar1), "button-press-event",
		      GTK_SIGNAL_FUNC (sb_mdown),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (hscrollbar1), "button-release-event",
		      GTK_SIGNAL_FUNC (sb_mup),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (leds), "expose-event",
                      GTK_SIGNAL_FUNC (leds_exposed),
                      NULL);

  return ovplayer;
}

